--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

printf(_VERSION)

-- 注册PLAYER_ENTERING_WORLD事件，确保SetupMyPreferences在所有界面元素加载完毕后执行
local frame = CreateFrame("Frame")
frame:Hide()
frame:SetScript(
    "OnEvent",
    function(self, event, ...)
        print(event)
        setup_my_preferences()
    end
)
frame:RegisterEvent("PLAYER_ENTERING_WORLD")
-- frame:RegisterAllEvents()

function dummy()
    -- 空函数，很多Frame需要把SetPoint设置为空防止之后的改变
end

local position = {
    top_left = "TOPLEFT",
    top = "TOP",
    top_right = "TOPRIGHT",
    left = "LEFT",
    center = "CENTER",
    right = "RIGHT",
    bottom_left = "BOTTOMLEFT",
    bottom = "BOTTOM",
    bottom_right = "BOTTOMRIGHT"
}

function set_point(f, p, r, rp, x, y)
    f:ClearAllPoints()
    f:SetPoint(p, r, rp, x, y)
end

function set_point_forever(f, p, r, rp, x, y)
    -- 设置位置并阻止之后的改变
    set_point(f, p, r, rp, x, y)
    f.SetPoint = dummy
end

function hide_forever(f)
    f:Hide()
    f.Show = dummy
end

function resize_fit_children(f)
    -- 调整父窗体大小适配所有儿子，WOW的坐标系为左下方(0, 0)
    local top, left, right, bottom = {}, {}, {}, {}
    for _, child in ipairs({f:GetChildren()}) do
        -- print(child:GetName(), child:GetTop(), child:GetLeft(), child:GetRight(), child:GetBottom())
        table.insert(top, child:GetTop())
        table.insert(left, child:GetLeft())
        table.insert(right, child:GetRight())
        table.insert(bottom, child:GetBottom())
    end
    local width = math.max(unpack(right)) - math.min(unpack(left))
    local height = math.max(unpack(top)) - math.min(unpack(bottom))
    f:SetSize(width, height)
end

function print_children(f, lvl)
    local prefix = ""
    for i = 0, lvl do
        prefix = prefix .. "    "
    end
    local name = f:GetName()
    if name == nil then
        return
    end
    ConsoleAddMessage(prefix .. name)
    if f.GetChildren == nil then
        return
    end
    for _, child in ipairs({f:GetChildren()}) do
        print_children(child, lvl + 1)
    end
end

function setup_my_preferences()
    -- 基本设置
    local settings = {
        -- 全局设置
        scriptErrors = "1", -- 显示Lua错误
        ffxDeath = "1",
        ffxGlow = "1",
        ffxNether = "1",
        profanityFilter = "0", -- 不良语言过滤
        overrideArchive = "0", -- 反和谐
        -- 控制
        autoLootDefault = "1", -- 自动拾取
        -- 名字
        nameplateShowAll = "1", -- 显示所有姓名板
        nameplateShowEnemyMinus = "0", -- 不显示杂兵姓名板
        -- 团队界面配置
        useCompactPartyFrames = "1", -- 团队风格小队框体
        -- raidOptionShowBorders = "0", -- 显示边框?
        -- raidFramesDisplayPowerBars = "1", -- 显示能量条?
        -- raidFramesDisplayClassColor = "1", -- 显示职业颜色?
        -- raidOptionDisplayPets = "1", -- 显示宠物?
        -- raidFramesHealthText = "health", -- 显示剩余生命值?
        -- 地图
        minimapTrackedInfov2 = "517160", -- 小地图追踪目标
        showTamers = "0", -- 地图显示宠物对战训练师
        questLogOpen = "0", -- 地图右侧显示任务
        mapFade = "0", -- 移动时地图透明
        -- 其它
        nameplateMaxDistance = "40" -- 姓名板距离
    }
    for k, v in pairs(settings) do
        -- print(k, v)
        SetCVar(k, v)
    end
    -- 反向整理背包
    SetSortBagsRightToLeft(true)
    -- 显示额外四个动作条，下一次生效
    SetActionBarToggles(1, 1, 1, 1, null)
    -- 以下代码立即生效
    SHOW_MULTI_ACTIONBAR_1 = 1
    SHOW_MULTI_ACTIONBAR_2 = 1
    SHOW_MULTI_ACTIONBAR_3 = 1
    SHOW_MULTI_ACTIONBAR_4 = 1
    InterfaceOptions_UpdateMultiActionBars()

    -- 设置界面元素位置，/fstack命令可以查看界面元素
    -- 姓名板位置
    set_point_forever(PlayerFrame, position.center, UIParent, position.center, -240, -30)
    set_point_forever(TargetFrame, position.center, UIParent, position.center, 240, -30)

    -- 将右侧两个动作条改为水平
    -- local bar_width = MultiBarBottomLeft:GetWidth()
    -- local bar_height = MultiBarBottomLeft:GetHeight()
    -- for i, v in ipairs {"Left", "Right"} do
    --     local bar_name = "MultiBar" .. v
    --     local bar = _G[bar_name]
    --     local button_name_prefix = bar_name .. "Button"
    --     -- 第1个按钮
    --     local button = _G[button_name_prefix .. 1]
    --     set_point_forever(button, position.bottom_left, bar_name, position.bottom_left, 0, 0)
    --     -- 之后11个按钮
    --     for i = 2, 12 do
    --         button = _G[button_name_prefix .. i]
    --         set_point_forever(button, position.left, button_name_prefix .. i - 1, position.right, 6, 0)
    --     end
    --     resize_fit_children(bar)
    -- end

    -- 添加主动作条
    local main_action_bar = CreateFrame("Frame", "main_action_bar")
    -- for i = 1, 12 do
    --     _G["ActionButton" .. i]:SetParent(main_action_bar)
    -- end
    resize_fit_children(main_action_bar) -- 必须先resize，否则获取ActionButton1的坐标就会失败（四个值都是nil）
    set_point_forever(ActionButton1, position.bottom_left, main_action_bar, position.bottom_left, 0, 0)
    main_action_bar:SetParent(MainMenuBarArtFrame)

    -- 添加命令按钮条
    -- local command_bar = CreateFrame("Frame", "command_bar")
    -- for _, button in ipairs(
    --     {
    --         CharacterMicroButton,
    --         SpellbookMicroButton,
    --         TalentMicroButton,
    --         AchievementMicroButton,
    --         QuestLogMicroButton,
    --         GuildMicroButton,
    --         LFDMicroButton,
    --         CollectionsMicroButton,
    --         EJMicroButton,
    --         StoreMicroButton,
    --         MainMenuMicroButton
    --     }
    -- ) do
    --     button:SetParent(command_bar)
    -- end
    -- resize_fit_children(command_bar)
    -- set_point(CharacterMicroButton, position.bottom_left, command_bar, position.bottom_left, 0, 0)

    -- 添加背包条
    -- local bag_bar = CreateFrame("Frame", "bag_bar")
    -- for _, button in ipairs(
    --     {
    --         MainMenuBarBackpackButton,
    --         CharacterBag0Slot,
    --         CharacterBag1Slot,
    --         CharacterBag2Slot,
    --         CharacterBag3Slot
    --     }
    -- ) do
    --     button:SetParent(bag_bar)
    -- end
    -- resize_fit_children(bag_bar)
    -- set_point_forever(MainMenuBarBackpackButton, position.bottom_right, bag_bar, position.bottom_right, 0, 0)

    -- 从MainMenuBarArtFrame中剥离无用元素
    -- for _, frame in ipairs(
    --     {
    --         MultiBarLeft,
    --         MultiBarRight,
    --         MultiBarBottomRight,
    --         MultiBarBottomLeft,
    --         ActionBarUpButton,
    --         ActionBarDownButton,
    -- main_action_bar,
    --         command_bar,
    --         bag_bar
    --     }
    -- ) do
    --     frame:SetParent(MainMenuBar)
    -- end

    -- MiniButton很特殊，不能剥离MainMenuBarArtFrame，否则进入退出载具界面会失效
    -- MainMenuBarOverlayFrame:Hide()
    MainMenuBarLeftEndCap:Hide()
    MainMenuBarRightEndCap:Hide()
    MainMenuBarTexture0:Hide()
    MainMenuBarTexture1:Hide()
    -- MainMenuBarTexture2:Hide()
    -- MainMenuBarTexture3:Hide()
    -- ActionBarUpButton:Hide()
    -- ActionBarDownButton:Hide()
    -- MainMenuBarPageNumber:Hide()
    -- set_point(CharacterMicroButton, position.bottom_left, MainMenuBarArtFrame, position.bottom_left, 0, 0)
    -- print_children(MainMenuBarArtFrame, 0)
    -- resize_fit_children(MainMenuBarArtFrame)

    -- 设置动作条位置
    -- set_point_forever(MultiBarRight, position.center, UIParent, position.center, 0, -200)
    -- set_point_forever(MultiBarLeft, position.top, MultiBarRight, position.bottom, 0, -4)
    -- set_point_forever(MultiBarBottomRight, position.bottom, MultiBarRight, position.top, 0, 4)
    -- set_point_forever(main_action_bar, position.center, UIParent, position.center, 0, -200)
    -- set_point_forever(MultiBarBottomLeft, position.top, main_action_bar, position.bottom, 0, -4)
    -- set_point_forever(command_bar, position.bottom_left, MultiBarLeft, position.bottom_right, 4, 0)
    -- set_point_forever(bag_bar, position.bottom_right, command_bar, position.top_right, 0, -20)
    -- resize_fit_children(PetActionBarFrame)
    -- set_point_forever(PetActionBarFrame, position.bottom_right, MultiBarLeft, position.bottom_left, -4, 0)

    -- 隐藏其它
    -- MainMenuBar:Hide() -- MainMenuBar很特殊，这里是无法隐藏的，会在屏幕上占一个透明的位置，但是如果执行的话，会把命令按钮都隐藏掉。据说和载具界面相关
    -- MainMenuBarArtFrame:Hide()
    -- resize_fit_children(MainMenuBarArtFrame)
    -- set_point_forever(MainMenuBarArtFrame, position.bottom_left, MultiBarLeft, position.bottom_right, 4, 0)
    -- hide_forever(MainMenuExpBar)
end

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

-- raidOptionShowBorders = "0", -- 显示边框?
-- raidFramesDisplayPowerBars = "1", -- 显示能量条?
-- raidFramesDisplayClassColor = "1", -- 显示职业颜色?
-- raidOptionDisplayPets = "1", -- 显示宠物?
-- raidFramesHealthText = "health", -- 显示剩余生命值?
-- minimapTrackedInfov2 = "517160", -- 小地图追踪目标

-- 设置透明度
-- for _, obj in ipairs(objs_to_adjust_transparency) do
--     -- print(obj:GetName())
--     obj:SetAlpha(0.7)
-- end

-- set_point_forever(BattlefieldMinimap, position.center, UIParent, position.center, 0, 0)
-- BattlefieldMinimap:SetFrameStrata("BACKGROUND")
-- BattlefieldMinimap:SetScale(2)
-- BattlefieldMinimap:EnableMouse(false)

-- PlayerFrameTexture:Hide()
-- PlayerFrameBackground:Hide()
-- PlayerFrameAlternateManaBarBackground:Hide()
-- PlayerFrameAlternateManaBarBorder:Hide()
-- PlayerFrameAlternateManaBarLeftBorder:Hide()
-- PlayerFrameAlternateManaBarRightBorder:Hide()
-- PetFrameTexture:Hide()
-- TargetFrameTextureFrameTexture:Hide()
-- TargetFrameBackground:Hide()
-- TargetFrameNameBackground:Hide() -- 不起作用
-- TargetFrameToTTextureFrameTexture:Hide()
-- TargetFrameToTBackground:Hide()

local btn = CreateFrame("Button", "myButton", UIParent, "SecureActionButtonTemplate")
btn:SetPoint(position.top_left, UIParent, position.top_left, 10, 10)
btn:SetSize(300, 100)
btn:SetText("AAAAAAAA")
btn:SetAttribute("type", "target")
btn:SetAttribute("unit", "player")
btn:SetBackdrop(
    {
        bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
        insets = {top = 0, left = 0, bottom = 0, right = 0}
    }
)
btn:SetBackdropColor(1, 1, 1, 0)
btn:SetNormalFontObject(GameFontNormal)
btn:SetNormalTexture("Interface/Buttons/UI-Panel-Button-Up")
btn:SetHighlightTexture("Interface/Buttons/UI-Panel-Button-Highlight")
btn:SetPushedTexture("Interface/Buttons/UI-Panel-Button-Down")
print(btn:GetText())
btn:SetHighlightTexture(ActionButton1:GetHighlightTexture():GetTexture())
btn:SetPushedTexture(ActionButton1:GetPushedTexture():GetTexture())

-- unpack必须放在最后
-- https://stackoverflow.com/questions/37372182/what-is-happening-when-i-call-unpack-as-luas-function-arguments
local objs_to_adjust_transparency = {PlayerFrame, TargetFrame, MultiBarBottomLeft, unpack(action_buttons)}

register_event(event.unit_exited_vehicle, set_player_target_frame_position) -- 貌似不起作用

InspectAchievements(uid) -- 打开比较成就面板

register_event(
    "COMBAT_LOG_EVENT_UNFILTERED",
    function()
        local _, _, _, guid, _, _, _, _, _, _, _ = CombatLogGetCurrentEventInfo()
        local class, _, race, _, gender, name, realm = GetPlayerInfoByGUID(guid)
        if class == nil then
            return
        end
        gender = (gender == 2) and "男" or "女"
        realm = (realm == "") and GetRealmName()
        print(name, class, race, gender, realm)
    end
)

local btn = CreateFrame("Button", "testButton", UIParent, "UIPanelButtonTemplate")
btn:SetText("测试")
btn:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", 10, 10)
btn:SetScript(
    "OnClick",
    function()
        local uid = "target"
        print(UnitGUID(uid))
        print(UnitHonorLevel(uid))
        print(SetAchievementComparisonUnit(uid))
    end
)

CompactRaidFrameManager_SetSetting("DisplayPets", true)

DEFAULT_CHAT_FRAME:SetClampedToScreen(false)
set_point(DEFAULT_CHAT_FRAME, "TOPLEFT", UIParent, "TOPLEFT", 30, -60)
DEFAULT_CHAT_FRAME:SetSize(500, 150)
DEFAULT_CHAT_FRAME.ClearAllPoints = function()
end
DEFAULT_CHAT_FRAME.SetPoint = function()
end

local r, g, b, _ = GetClassColor(cid)
btn:GetFontString():SetTextColor(r, g, b, 1)

local action_buttons = {
    ActionButton1,
    ActionButton2,
    ActionButton3,
    ActionButton4,
    ActionButton5,
    ActionButton6,
    ActionButton7,
    ActionButton8,
    ActionButton9,
    ActionButton10,
    ActionButton11,
    ActionButton12
}

local multi_bar_bottom_left_buttons = {
    MultiBarBottomLeftButton1,
    MultiBarBottomLeftButton2,
    MultiBarBottomLeftButton3,
    MultiBarBottomLeftButton4,
    MultiBarBottomLeftButton5,
    MultiBarBottomLeftButton6,
    MultiBarBottomLeftButton7,
    MultiBarBottomLeftButton8,
    MultiBarBottomLeftButton9,
    MultiBarBottomLeftButton10,
    MultiBarBottomLeftButton11,
    MultiBarBottomLeftButton12
}

local multi_bar_bottom_right_buttons = {
    MultiBarBottomRightButton1,
    MultiBarBottomRightButton2,
    MultiBarBottomRightButton3,
    MultiBarBottomRightButton4,
    MultiBarBottomRightButton5,
    MultiBarBottomRightButton6,
    MultiBarBottomRightButton7,
    MultiBarBottomRightButton8,
    MultiBarBottomRightButton9,
    MultiBarBottomRightButton10,
    MultiBarBottomRightButton11,
    MultiBarBottomRightButton12
}

-- 动作按钮条
local action_bar = CreateFrame("Frame", nil)
local multi_bar_bottom_left = CreateFrame("Frame", nil) -- MultiBarBottomLeft会重设与ActionButton1的位置
local multi_bar_bottom_right = CreateFrame("Frame", nil)

function setup_action_bars()
    -- print("setup_action_bars")

    set_point(ActionButton7, "LEFT", ActionButton6, "RIGHT", 28, 0)
    set_point(MultiBarBottomLeftButton7, "LEFT", MultiBarBottomLeftButton6, "RIGHT", 28, 0)

    -- 注意千万不能改变这些控件的父子关系，否则会有很多奇怪的问题，比如小德形态切换ActionButton功能丢失，载具上下后MiniButton失效
    action_bar:SetParent(MainMenuBarArtFrame)
    action_bar:SetSize(get_size(unpack(action_buttons)))
    multi_bar_bottom_left:SetParent(MainMenuBarArtFrame)
    multi_bar_bottom_left:SetSize(get_size(unpack(multi_bar_bottom_left_buttons)))
    multi_bar_bottom_right:SetParent(MainMenuBarArtFrame)
    multi_bar_bottom_right:SetSize(get_size(unpack(multi_bar_bottom_right_buttons)))

    -- 设置位置
    set_point(ActionButton1, "BOTTOMLEFT", action_bar, "BOTTOMLEFT", 0, 0)
    set_point(action_bar, "CENTER", UIParent, "CENTER", 0, -300)
    set_point(MultiBarBottomLeftButton1, "BOTTOMLEFT", multi_bar_bottom_left, "BOTTOMLEFT", 0, 0)
    set_point(multi_bar_bottom_left, "TOP", action_bar, "BOTTOM", 0, -28)
    set_point(MultiBarBottomRightButton1, "BOTTOMLEFT", multi_bar_bottom_right, "BOTTOMLEFT", 0, 0)
    set_point(multi_bar_bottom_right, "BOTTOMRIGHT", MicroButtonAndBagsBar, "BOTTOMLEFT", -10, 0)

    -- 透明度
    -- action_bar:SetAlpha(0.6)
    -- multi_bar_bottom_left:SetAlpha(0.6)

    -- 小地图
    -- set_point(MinimapCluster, "TOP", UIParent, "TOP", 0, 0)
    -- MinimapCluster:SetFrameStrata("BACKGROUND")
    -- set_point(Minimap, "CENTER", UIParent, "CENTER", 0, 0)
    -- Minimap:SetClampedToScreen(false)
    -- Minimap:SetFrameStrata("BACKGROUND")
    -- MinimapCluster:SetAlpha(0.2)
    -- Minimap:SetScale(3)

    -- 任务列表
    -- https://us.battle.net/forums/en/wow/topic/15141304174#2
    -- local me_moving = false -- 防止无限循环调用
    -- hooksecurefunc(
    --     ObjectiveTrackerFrame,
    --     "SetPoint",
    --     function()
    --         if me_moving then
    --             return
    --         end
    --         me_moving = true
    --         set_point(ObjectiveTrackerFrame, "TOPRIGHT", UIParent, "TOPRIGHT", 0, 0)
    --         me_moving = false
    --     end
    -- )

    -- DEFAULT_CHAT_FRAME:SetClampedToScreen(false)
    -- set_point(DEFAULT_CHAT_FRAME, "BOTTOMLEFT", UIParent, "BOTTOMLEFT", 30, 0)
    -- DEFAULT_CHAT_FRAME.ClearAllPoints = function()
    -- end
    -- DEFAULT_CHAT_FRAME.SetPoint = function()
    -- end
end

-- local btn =
--     create_button(
--     "查找当前地图所有玩家",
--     function()
--         local zone = C_Map.GetMapInfo(C_Map.GetBestMapForUnit("player")).name
--         local filter = string.format('z-"%s" 1-120', zone)
--         SetWhoToUI(1)
--         SendWho(filter)
--     end
-- )
-- btn:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", 0, 20)

-- register_event(
--     "WHO_LIST_UPDATE",
--     function()
--         local numWhos, totalCount = GetNumWhoResults()
--         for i = 1, numWhos do
--             print_chat(GetWhoInfo(i))
--         end
--     end
-- )
-- register_event("CHAT_MSG_ADDON", print_chat)

--测试
-- https://us.battle.net/forums/en/wow/topic/4081817152
-- https://www.mmo-champion.com/threads/1372812-LUA-How-to-get-tooltips
-- https://us.battle.net/forums/en/wow/topic/3424470486

-- register_event(
--     "SPELL_UPDATE_COOLDOWN",
--     function(...)
--         print_chat("SPELL_UPDATE_COOLDOWN", ...)
--     end
-- )
-- register_event(
--     "BAG_UPDATE_COOLDOWN",
--     function(...)
--         print_chat("BAG_UPDATE_COOLDOWN", ...)
--     end
-- )
-- register_event(
--     "ACTIONBAR_UPDATE_COOLDOWN",
--     function(...)
--         print_chat("ACTIONBAR_UPDATE_COOLDOWN", ...)
--     end
-- )
-- register_event(
--     "UPDATE_SHAPESHIFT_COOLDOWN",
--     function(...)
--         print_chat("UPDATE_SHAPESHIFT_COOLDOWN", ...)
--     end
-- )

-- local btn = CreateFrame("Button", nil, UIParent, "ActionButtonTemplate")
-- btn:SetAttribute("type", "item")
-- btn:SetAttribute("item", "狩猎帽")
-- btn:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 10, -10)
-- btn.icon:SetTexture(GetItemIcon(92738))
-- btn:RegisterForDrag("LeftButton", "RightButton")
-- btn:SetScript(
--     "OnEnter",
--     function(self)
--         GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
--         GameTooltip:SetItemByID(92738)
--         GameTooltip:Show()
--     end
-- )
-- btn:SetScript(
--     "OnLeave",
--     function(self)
--         GameTooltip:Hide()
--     end
-- )
-- btn:SetScript(
--     "OnReceiveDrag",
--     function(self)
--         print_chat(GetCursorInfo())
--         ClearCursor()
--     end
-- )

-- btn.spellID = "92738"
-- local myCooldown = CreateFrame("Cooldown", nil, btn, "CooldownFrameTemplate")
-- myCooldown:SetAllPoints()
-- myCooldown:SetCooldown(GetTime(), 10)

--     local button =
--         DragExample or
--         CreateFrame(
--             "Button",
--             "DragExample",
--             UIParent,
--             "ActionButtonTemplate,SecureActionButtonTemplate,SecureHandlerDragTemplate"
--         )
--     button:SetWidth(36)
--     button:SetHeight(36)
--     button:SetPoint("TOPLEFT")

--     button:SetAttribute(
--         "_onreceivedrag",
--         [[
--         -- kind,value,... are returns of GetCursorInfo()
--         if kind=="item" or kind=="spell" or kind=="macro" then
--             self:SetAttribute("type",kind)
--             if kind=="item" then
--             value = "item:"..value
--             elseif kind=="spell" then
--             value = select(2,...) -- the raw spell id (4th return GetCursorInfo)
--             end
--             -- "item","item:1234" or "spell",1234 or "macro",1234
--             self:SetAttribute(kind,value)
--             return "clear"
--         end
--         ]]
--     )

--     button:SetScript(
--         "OnAttributeChanged",
--         function(self, attribType, attribDetail)
--             -- only changing texture. not dealing with cooldowns or item counts
--             if attribType == "item" then
--                 self.icon:SetTexture((select(10, GetItemInfo(attribDetail))))
--             elseif attribType == "spell" then
--                 self.icon:SetTexture((select(3, GetSpellInfo(attribDetail))))
--             elseif attribType == "macro" then
--                 self.icon:SetTexture((select(2, GetMacroInfo(attribDetail))))
--             end
--         end
--     )

-- local button =
--     DragExample2 or
--     CreateFrame("Button", "DragExample2", UIParent, "ActionButtonTemplate,SecureActionButtonTemplate")
-- button:SetWidth(36)
-- button:SetHeight(36)
-- button:SetPoint("LEFTTOP")

-- local validTypes = {item = 1, spell = 1, macro = 1}

-- local function defineButton(self)
--     local cursorType, id, _, spell = GetCursorInfo()
--     ClearCursor()

--     if InCombatLockdown() then
--         return -- can't do this stuff in combat
--     end

--     local texture
--     if cursorType == "item" then
--         id = "item:" .. id
--         texture = select(10, GetItemInfo(id))
--     elseif cursorType == "spell" then
--         id = spell
--         texture = select(3, GetSpellInfo(id))
--     elseif cursorType == "macro" then
--         _, texture = GetMacroInfo(id)
--     end
--     self:SetAttribute("type", cursorType)
--     self:SetAttribute(cursorType, id)
--     self.icon:SetTexture(texture)
-- end

-- -- There's likely a better way, but this is a cheap way to prevent
-- -- a "click" on a button when dropping something onto it.
-- -- If the cursor contains a valid object, nil the button's type...
-- button:SetScript(
--     "PreClick",
--     function(self)
--         if validTypes[GetCursorInfo()] and not InCombatLockdown() then
--             self:SetAttribute("type", nil)
--         end
--     end
-- )
-- -- ...and in the PostClick, define the button
-- button:SetScript(
--     "PostClick",
--     function(self)
--         if validTypes[GetCursorInfo()] then
--             defineButton(self)
--         end
--     end
-- )
-- button:SetScript("OnReceiveDrag", defineButton)

local RACIAL_ABILITY = {
    [1] = 59752, -- 人类
    [2] = 20572, -- 兽人
    [3] = 20594, -- 矮人
    [4] = 58984, -- 暗夜精灵
    [5] = 0, -- 亡灵
    [6] = 20549, -- 牛头人
    [7] = 20589, -- 侏儒
    [8] = 0, -- 巨魔
    [9] = 69070, -- 地精
    [10] = 28730, -- 血精灵
    [11] = 28880, -- 德莱尼
    [22] = 68992, -- 狼人
    [24] = 107079, -- 熊猫人
    [25] = 107079, -- 熊猫人
    [26] = 107079, -- 熊猫人
    [27] = 0, -- 夜之子
    [28] = 0, -- 高岭牛头人
    [29] = 256948, -- 虚空精灵
    [30] = 255647, -- 光铸德莱尼
    [31] = 0, -- 赞达拉巨魔
    [32] = 287712, -- 库尔提拉斯人
    [34] = 265221, -- 黑铁矮人
    [36] = 0 -- 玛格汉兽人
}
for rid, sid in pairs(RACIAL_ABILITY) do
    rinfo = C_CreatureInfo.GetRaceInfo(rid)
    finfo = C_CreatureInfo.GetFactionInfo(rid)
    sname = GetSpellInfo(sid)
    sdesc = GetSpellDescription(sid)
    print(rinfo.raceName, sname, sdesc)
end

function foo()
    T, F = T or 0, F or CreateFrame("frame")
    if X then
        X = nil
        SendSystemMessage("防离关闭")
    else
        SendSystemMessage("防离开启")
        X = function()
            local t = GetTime()
            if t - T > 1 then
                StaticPopup1Button2:Click()
                T = t
            end
        end
    end
    F:SetScript("OnUpdate", X)
end

function setup_action_bars()
    local s_gap = 4
    local l_x_gap = 40
    local l_y_gap = 12
    local y_offset = 300
    -- 原始的普通按钮宽度36，宠物按钮30
    local l_scale = 1
    local s_scale = l_scale * 5 / 6
    local l_alpha = 1
    local s_alpha = l_alpha * 0.6
    local btn_prefix_list = {
        "ActionButton",
        "MultiBarBottomLeftButton",
        "MultiBarBottomRightButton",
        "MultiBarLeftButton",
        "MultiBarRightButton"
    }

    -- 动作条
    for i, prefix in ipairs(btn_prefix_list) do
        -- 按钮6、7的位置
        if i == 1 then
            set_point(ActionButton6, "TOPRIGHT", UIParent, "CENTER", -l_x_gap / 2, -y_offset)
            set_point(ActionButton7, "TOPLEFT", UIParent, "CENTER", l_x_gap / 2, -y_offset)
        else
            local gap = (i == 3) and l_y_gap or s_gap
            set_point(_G[prefix .. 6], "TOPRIGHT", _G[btn_prefix_list[i - 1] .. 6], "BOTTOMRIGHT", 0, -gap)
            set_point(_G[prefix .. 7], "TOPLEFT", _G[btn_prefix_list[i - 1] .. 7], "BOTTOMLEFT", 0, -gap)
        end
        -- 其它按钮的位置
        for num = 5, 1, -1 do
            set_point(_G[prefix .. num], "RIGHT", _G[prefix .. (num + 1)], "LEFT", -s_gap, 0)
        end
        for num = 8, 12 do
            set_point(_G[prefix .. num], "LEFT", _G[prefix .. (num - 1)], "RIGHT", s_gap, 0)
        end
        -- 缩小按钮
        for num = 1, 12 do
            local scale = (i <= 2) and l_scale or s_scale
            local alpha = (i <= 2) and l_alpha or s_alpha
            _G[prefix .. num]:SetScale(scale)
            _G[prefix .. num]:SetAlpha(alpha)
        end
    end
    set_point(ActionBarUpButton, "BOTTOMLEFT", ActionButton12, "RIGHT", 0, 0)
    set_point(ActionBarDownButton, "TOPLEFT", ActionButton12, "RIGHT", 0, 0)
    VerticalMultiBarsContainer:SetParent(MainMenuBarArtFrame) -- 否则最下面一排会被MainMenuBar覆盖
    -- MultiBarLeft、MultiBarRight的Scale会随着Minimap和ObjectiveTrackerFrame而变化，会影响到其中按钮的大小
    hook_non_recursive(
        MultiBarLeft,
        "SetScale",
        function()
            MultiBarLeft:SetScale(1)
        end
    )
    hook_non_recursive(
        MultiBarRight,
        "SetScale",
        function()
            MultiBarRight:SetScale(1)
        end
    )

    -- 宠物条，姿态条
    for num = 1, 10 do
        _G["PetActionButton" .. num]:SetScale(s_scale)
        _G["PetActionButton" .. num]:SetAlpha(s_alpha)
        _G["StanceButton" .. num]:SetScale(s_scale)
        _G["StanceButton" .. num]:SetAlpha(s_alpha)
        if num >= 2 then
            set_point(_G["PetActionButton" .. num], "LEFT", _G["PetActionButton" .. (num - 1)], "RIGHT", s_gap, 0)
            set_point(_G["StanceButton" .. num], "LEFT", _G["StanceButton" .. (num - 1)], "RIGHT", s_gap, 0)
        end
    end
    hook_set_point(
        PetActionBarFrame,
        function()
            set_point(PetActionButton1, "BOTTOMLEFT", ActionButton1, "TOPLEFT", 0, l_y_gap)
        end
    )
    hook_set_point(
        StanceBarFrame,
        function()
            set_point(StanceButton1, "BOTTOMLEFT", ActionButton1, "TOPLEFT", 0, l_y_gap)
        end
    )

    -- 其它
    MainMenuBarArtFrame.LeftEndCap:Hide()
    MainMenuBarArtFrame.RightEndCap:Hide()
    MainMenuBarArtFrameBackground:Hide()
    MainMenuBarArtFrame.PageNumber:Hide()
end

function setup_player_target_frame()
    local x_offset = 360
    local y_offset = 140
    local scale = 1
    local buff_scale = scale * 0.8

    -- hook_set_point(
    --     PlayerFrame,
    --     function()
    --         if not UnitHasVehicleUI("player") then
    --             set_point(PlayerFrame, "TOPRIGHT", UIParent, "CENTER", -x_offset, -y_offset)
    --             PlayerFrame:SetScale(scale)
    --         end
    --     end
    -- )
    -- hook_set_point(
    --     TargetFrame,
    --     function()
    --         if not UnitHasVehicleUI("player") then
    --             set_point(TargetFrame, "TOPLEFT", UIParent, "CENTER", x_offset, -y_offset)
    --             TargetFrame:SetScale(scale)
    --         end
    --     end
    -- )
    BuffFrame:SetScale(buff_scale)
    hook_set_point(
        BuffFrame,
        function()
            set_point(BuffFrame, "TOPRIGHT", PlayerFrame, "TOPLEFT", 0, 20)
        end
    )

    -- 这样设置PlayerFrame和TargetFrame才对
    -- https://us.battle.net/forums/en/wow/topic/6572970645
    -- https://github.com/tomrus88/BlizzardInterfaceCode/blob/master/Interface/FrameXML/PlayerFrame.lua
    -- https://github.com/tomrus88/BlizzardInterfaceCode/blob/master/Interface/FrameXML/TargetFrame.lua
    -- https://github.com/tomrus88/BlizzardInterfaceCode/blob/master/Interface/FrameXML/UIParent.lua
    local function set_player_target_frame_position()
        -- print("set_player_target_frame_position")
        if (not UnitHasVehicleUI("player")) and (not InCombatLockdown()) then
            set_point(PlayerFrame, "TOPRIGHT", UIParent, "CENTER", -x_offset, -y_offset)
            set_point(TargetFrame, "TOPLEFT", UIParent, "CENTER", x_offset, -y_offset)
        end
    end
    hooksecurefunc("UIParent_UpdateTopFramePositions", set_player_target_frame_position)
    hooksecurefunc("PlayerFrame_UpdateArt", set_player_target_frame_position)
    hooksecurefunc("PlayerFrame_AnimFinished", set_player_target_frame_position)
    hooksecurefunc("PlayerFrame_SequenceFinished", set_player_target_frame_position)

    -- 屏幕正中间显示十字
    local f = CreateFrame("Frame", nil, UIParent)
    f:SetFrameStrata("BACKGROUND")
    f:SetSize(256, 256)
    f:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
    local t = f:CreateTexture(nil, "BACKGROUND")
    t:SetTexture("Interface/AddOns/_MyPreference/res/cross.tga")
    t:SetAlpha(0.15)
    t:SetAllPoints()
end

function setup_chat_frame()
    hook_set_point(
        DEFAULT_CHAT_FRAME,
        function()
            set_point(DEFAULT_CHAT_FRAME, "BOTTOMLEFT", UIParent, "BOTTOMLEFT", 0, 80)
        end
    )
end

function setup_minimap()
    -- 小地图
    MinimapCluster:SetScale(1.2)
    MinimapBorderTop:Hide()
    MinimapBorder:Hide()
    -- Minimap:SetBackdrop(
    --     {
    --         bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark",
    --         edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
    --         edgeSize = 12
    --     }
    -- )
    Minimap:SetMaskTexture("Interface/ChatFrame/ChatFrameBackground")
    Minimap:SetPlayerTexture("Interface/AddOns/_MyPreference/res/minimap_arrow_3.tga")

    -- 从Chinchilla Minimap插件里拷过来的，默认是Interface\\MiniMap\\ObjectIconsAtlas
    -- Minimap:SetBlipTexture("Interface/AddOns/_MyPreference/res/object_icons.tga")
    -- 挪动位置会造成非常多的未知错误
    -- set_point(MinimapCluster, "BOTTOMRIGHT", MicroButtonAndBagsBar, "BOTTOMLEFT", 0, 0)

    -- BattlefieldMapFrame默认为nil，hooksecurefunc会调用失败
    -- https://us.forums.blizzard.com/en/wow/t/simple-macro-but-nil-value-as-an-addon/59214/2
    BattlefieldMap_LoadUI()
    hook_set_point(
        BattlefieldMapFrame,
        function()
            set_point(BattlefieldMapFrame, "BOTTOMRIGHT", MicroButtonAndBagsBar, "TOPRIGHT", 0, 6)
        end
    )

    -- 任务列表
    hook_set_point(
        ObjectiveTrackerFrame,
        function()
            ObjectiveTrackerFrame:ClearAllPoints()
            ObjectiveTrackerFrame:SetPoint("TOPRIGHT", MinimapCluster, "BOTTOMRIGHT", 0, 0)
            ObjectiveTrackerFrame:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", 0, 100)
        end
    )
end

-- register_event(
--     "PLAYER_ENTERING_WORLD",
--     function()
--         -- 显示额外四个动作条，下一次生效
--         SetActionBarToggles(1, 1, 1, 1, 1)
--         -- 以下代码立即生效
--         SHOW_MULTI_ACTIONBAR_1 = 1
--         SHOW_MULTI_ACTIONBAR_2 = 1
--         SHOW_MULTI_ACTIONBAR_3 = 1
--         SHOW_MULTI_ACTIONBAR_4 = 1
--         MultiActionBar_Update()
--         print(SHOW_MULTI_ACTIONBAR_1)
--     end
-- )

-- 隐藏拾取弹出框
-- https://github.com/Gethe/wow-ui-source/blob/live/FrameXML/AlertFrames.lua

-- 小宠物聊天
SLASH_PETCHAT1 = "/pc"
SlashCmdList.PETCHAT = function(msg)
    C_PetJournal.SetCustomName(C_PetJournal.GetSummonedPetGUID(), msg)
    C_Timer.After(
        1,
        function()
            if string.lower(GetUnitName("target")) == string.lower(msg) then
                DoEmote("DANCE")
            else
                print_chat("|cFFFF9900小宠物聊天失败")
            end
        end
    )
end

-- 法力值警告
local mana = 0
local mana_max = 0
local warned = false
local function warn_on_low()
    -- print_chat(mana, mana_max)
    if mana < (mana_max * 0.1) then
        if not warned then
            SendChatMessage("警告：法力值低于10%")
            warned = true
        end
    else
        warned = false
    end
end
local function refresh_mana()
    mana = UnitPower("player", Enum.PowerType.Mana)
    warn_on_low()
end
local function refresh_mana_max()
    mana_max = UnitPowerMax("player", Enum.PowerType.Mana)
    warn_on_low()
end
refresh_mana()
refresh_mana_max()
register_event(
    "UNIT_POWER_UPDATE",
    function(arg1, arg2)
        if arg1 == "player" and arg2 == "MANA" then
            refresh_mana()
        end
    end
)
register_event(
    "UNIT_MAXPOWER",
    function(arg1, arg2)
        if arg1 == "player" and arg2 == "MANA" then
            refresh_mana_max()
        end
    end
)


    -- 世界地图
    -- 地图 SimpleMap插件中拷贝
    WorldMapFrame.ScrollContainer.GetCursorPosition = function(f)
        local x, y = MapCanvasScrollControllerMixin.GetCursorPosition(f)
        local s = WorldMapFrame:GetScale()
        return x / s, y / s
    end
    WorldMapFrame.BlackoutFrame.Blackout:SetAlpha(0)
    WorldMapFrame.BlackoutFrame:EnableMouse(false)
    local function scale_world_map()
        print("WorldMapFrame:SetScale(0.5)")
        WorldMapFrame:SetScale(0.5)
    end
    WorldMapFrame:HookScript("OnShow", scale_world_map) -- 不管用
    WorldMapFrame.BorderFrame.MaximizeMinimizeFrame.MaximizeButton:HookScript("OnClick", scale_world_map)
    WorldMapFrame.BorderFrame.MaximizeMinimizeFrame.MinimizeButton:HookScript("OnClick", scale_world_map)


        -- 反和谐，貌似SetCVar不管用
        ConsoleExec("portal TW")
        ConsoleExec("profanityFilter 0")
        -- 以上，ConsoleExec也可以用SlashCmdList["CONSOLE"]替代
    
    -- 隐藏错误框并将其中的信息移到聊天框中
    UIErrorsFrame:Hide()
    hooksecurefunc(
        UIErrorsFrame,
        "AddMessage",
        function(self, txt, r, g, b, i, t)
            DEFAULT_CHAT_FRAME:AddMessage(txt, r, g, b, i, t)
        end
    )

    -- 隐藏任务对话框
    -- http://www.wowinterface.com/forums/showthread.php?t=55806
    -- local function HookTalkingHead()
    --     hooksecurefunc(
    --         "TalkingHeadFrame_PlayCurrent",
    --         function()
    --             TalkingHeadFrame:Hide()
    --         end
    --     )
    -- end
    -- if TalkingHeadFrame then
    --     HookTalkingHead()
    -- else -- FrameXML/UIParent.lua
    --     hooksecurefunc("TalkingHead_LoadUI", HookTalkingHead)
    -- end

        -- 角色信息按钮上弹出的黄边框提示信息，例如神器插槽等
    -- CharacterMicroButtonAlert:SetAlpha(0.2)

    -- 按键设置
    key_bindings = {}
    for i = 1, GetNumBindings() do
        key_bindings[i] = {GetBinding(i)}
    end

            -- 自动隐藏已完成任务
            -- QUEST_WATCH_UPDATE = function(event, quest_index)
            --     local title, _, _, _, _, is_complete, _, quest_id = GetQuestLogTitle(quest_index)
            --     local is_quest_complete = IsQuestComplete(quest_id)
            --     print(title, is_complete, is_quest_complete)
            --     if is_quest_complete then
            --         print("任务'" .. title .. "'已完成")
            --         RemoveQuestWatch(quest_index)
            --     end
            -- end,



    -- GameTooltip:HookScript(
    --     "OnTooltipSetUnit",
    --     function(self)
    --         local unit = select(2, self:GetUnit())
    --         print(unit)
    --     end
    -- )
