local _, T = ...

function T.Setup_Automation()
    local function player_string(name, realm, class)
        -- 玩家链接字符串格式：https://wow.gamepedia.com/PlayerString
        -- 对于realm相同，UnitName返回nil而GetPlayerInfoByGUID返回""
        local fname = (not realm or realm == "") and name or string.format("%s-%s", name, realm)
        local _, _, _, color = GetClassColor(class) -- 失败返回的color为ffffffff
        return string.format("|c%s|Hplayer:%s|h%s|h|r", color, fname, fname)
    end

    local function alter_friend_notes(name, ...)
        -- 如果是好友且好友备注匹配指定的模式则修改，参数同string.gsub
    end

    for k, v in pairs(
        {
            -- 自动修理、售卖垃圾
            MERCHANT_SHOW = function()
                if CanMerchantRepair() then
                    if IsInGuild() and CanGuildBankRepair() then
                        RepairAllItems(true)
                    end
                    RepairAllItems(false)
                end
                for bagID = 0, 4 do
                    local numberOfSlots = GetContainerNumSlots(bagID)
                    if numberOfSlots and numberOfSlots > 0 then
                        for slotIndex = 1, numberOfSlots do
                            local _, _, _, quality = GetContainerItemInfo(bagID, slotIndex)
                            if quality == 0 then
                                UseContainerItem(bagID, slotIndex)
                            end
                        end
                    end
                end
            end,
            -- 自动拒绝插旗
            DUEL_REQUESTED = CancelDuel,
            -- 成就完成自动截图
            -- ACHIEVEMENT_EARNED = Screenshot,
            -- 检查目标玩家
            PLAYER_TARGET_CHANGED = function()
                local uid = "target"
                -- if IsInInstance() then
                --     return
                -- end
                if not UnitIsPlayer(uid) then
                    return
                end
                local name, realm = UnitName(uid)
                local descr = player_string(name, realm, UnitClassBase(uid))
                descr = descr .. string.format(" 荣誉%d", UnitHonorLevel(uid))
                local guild, grank, gindex = GetGuildInfo(uid)
                if guild then
                    descr = descr .. string.format(" %s%d阶%s", guild, gindex, grank)
                end
                SendSystemMessage(descr)
                ClearAchievementComparisonUnit()
                SetAchievementComparisonUnit(uid) -- 继续比较成就
                -- 灵魂状态，组合键按住的情况下打开目标玩家面板
                -- if UnitIsGhost("player") and IsModifierKeyDown() then
                --     InspectUnit("target")
                -- end
            end,
            INSPECT_ACHIEVEMENT_READY = function(event, guid)
                local _, class, _, _, _, name, realm = GetPlayerInfoByGUID(guid)
                SendSystemMessage(
                    string.format("%s 成就%d", player_string(name, realm, class), GetComparisonAchievementPoints())
                )
            end,
            -- 自动释放灵魂
            PLAYER_DEAD = function()
                if not DeathRecap_HasEvents() then -- 使用帮助页面自杀没有死亡摘要，自动释放，其它情况不释放
                    RepopMe()
                end
            end,
            -- 自动隐藏已完成任务
            QUEST_LOG_UPDATE = function()
                -- print("QUEST_LOG_UPDATE")
                local to_be_removed = {}
                for watch_index = 1, C_QuestLog.GetNumQuestWatches() do
                    local quest_id = C_QuestLog.GetQuestIDForQuestWatchIndex(watch_index)
                    local is_complete = C_QuestLog.IsComplete(quest_id)
                    if is_complete then
                        table.insert(to_be_removed, quest_id)
                    end
                end
                for _, quest_id in ipairs(to_be_removed) do
                    C_QuestLog.RemoveQuestWatch(quest_id)
                end
            end
        }
    ) do
        T.Register_Event(k, v)
    end

    if T.IsAuthor then
        -- 阻止长时间没动作系统自动登出（同样也阻止手动登出）
        local anti_logout = {
            name = "ANTI_LOGOUT",
            commands = {"/al"}
        }
        T.Slash_Command_Default_Toggle_Handler(anti_logout, "Anti Logout", "state", true)
        T.Add_Slash_Command(anti_logout)
        T.Register_Event(
            "PLAYER_CAMPING",
            function()
                if anti_logout.state then
                    StaticPopup_Hide("CAMP")
                end
            end
        )
        T.Hook_StaticPopup(
            "CAMP",
            function(dialog) -- CAMP：返回角色选择对话框
                if anti_logout.state then
                    StaticPopup_Hide("CAMP")
                end
            end
        )
        ChatFrame_AddMessageEventFilter(
            "CHAT_MSG_SYSTEM",
            function(self, event, text, ...)
                -- 因长时间没有动作，将自动退出游戏。如果你希望继续游戏，请点击取消键。
                -- 你现在处于离开状态：离开
                -- 你不再处于离开状态。
                return text:find("因长时间没有动作") or text:find("处于离开状态")
            end
        )
    end

    -- 取消某些动画
    local old_mfpf = MovieFrame_PlayMovie
    MovieFrame_PlayMovie = function(self, movie_id)
        print("MovieFrame_PlayMovie", movie_id)
        if
            ({
                [16] = true, -- 阿尔萨斯死亡
                [189] = true, -- 部落1级要塞
                [190] = true, -- 部落2级要塞
                [192] = true, -- 联盟1级要塞
                [193] = true, -- 联盟2级要塞
                [294] = true, -- 阿克蒙德开始
                [295] = true, -- 阿克蒙德结束
                [549] = true, -- 古尔丹结束
                [678] = true, -- 维迪卡尔前往阿古斯
                [686] = true, -- 寂灭者阿古斯开始
                [688] = true, -- 寂灭者阿古斯结束
                [858] = true, -- 8.0引导动画
                [883] = true, -- 联盟纳沙塔尔引导任务
                [894] = true, -- 部落纳沙塔尔引导任务
                [937] = true, -- 9.0前夕引导任务动画
                [942] = true -- 9.0前夕舔狗死亡动画
            })[movie_id]
         then
            print("Calceled")
            GameMovieFinished()
            return
        else
            return old_mfpf(self, movie_id)
        end
    end

    for k, v in pairs(
        {
            Blizzard_TalkingHeadUI = function()
                -- 隐藏任务对话动画框
                -- TalkingHeadFrame:UnregisterAllEvents()
                -- TalkingHeadFrame.OnEvent = T.Nop
                -- TalkingHeadFrame:Click("RightButton")
                -- TalkingHeadFrame.MainFrame.CloseButton:Click()
                -- TalkingHeadFrame_CloseImmediately()
                hooksecurefunc("TalkingHeadFrame_PlayCurrent", TalkingHeadFrame_CloseImmediately)
                TalkingHeadFrame_CloseImmediately()
            end,
            Blizzard_ObjectiveTracker = function()
                -- 右侧场景任务结束的奖励闪现隐藏掉
                ScenarioObjectiveTracker_AnimateReward = T.Nop
                -- 世界任务屏幕中间提示去掉
                hooksecurefunc(
                    ObjectiveTrackerBonusBannerFrame.Anim,
                    "Play",
                    function()
                        ObjectiveTrackerBonusBannerFrame.Anim:Stop()
                        ObjectiveTrackerBonusBannerFrame:Hide()
                    end
                )
            end,
            Blizzard_GMChatUI = function()
                -- 自动点击GM对话提示（关闭提示并弹出GM对话框）
                hooksecurefunc(GMChatStatusFrame, "Show", GMChatStatusFrame.Click)
                -- hooksecurefunc(GMChatFrame, "Show", GMChatFrame.Hide)
            end,
            Blizzard_SubscriptionInterstitialUI = function()
                -- 试玩“即可订阅”页面
                hooksecurefunc(
                    SubscriptionInterstitialFrame,
                    "Show",
                    function()
                        SubscriptionInterstitialFrame.ClosePanelButton:Click()
                        SubscriptionInterstitialFrame.CloseButton:Click()
                    end
                )
                SubscriptionInterstitialFrame.ClosePanelButton:Click()
                SubscriptionInterstitialFrame.CloseButton:Click()
            end,
            Blizzard_CovenantToasts = function()
                -- 隐藏9.0名望升级屏幕中心提示
                CovenantRenownToast:UnregisterAllEvents()
                CovenantRenownToast.OnEvent = T.Nop
                T.Register_Event("COVENANT_SANCTUM_RENOWN_LEVEL_CHANGED", print)
            end
        }
    ) do
        T.On_AddOn_Loaded(k, v)
    end

    -- 隐藏团本BOSS击杀弹出框
    -- FrameXML/LevelUpDisplay.lua，BossBannerHider插件，http://nga.178.com/read.php?tid=10346095
    BossBanner:UnregisterAllEvents()
    BossBanner_OnLoad = T.Nop
    BossBanner.OnLoad = T.Nop
    BossBanner.OnEvent = T.Nop
    BossBanner_OnEvent = T.Nop

    -- 隐藏拾取弹出
    -- LootWonAlert_Hider_Mover插件
    -- xloot插件
    -- AlertFrameSystems.lua, AlertFrames.lua, LootFrame.lua
    -- LootAlertSystem.AddAlert = function()
    -- end
    AlertFrame:UnregisterAllEvents()
    AlertFrame.OnEvent = T.Nop
    -- AlertFrame.AddAlertFrame = function()end

    --  隐藏团本boss介绍
    -- EventToastManagerFrame:UnregisterAllEvents()
    -- function EventToastManagerFrame:OnEvent(event, ...)
    --     if (event == "DISPLAY_EVENT_TOASTS") then
    --         print("EventToastManagerFrame:OnEvent")
    --     end
    -- end
    -- EventToastManagerFrame:OnEvent("DISPLAY_EVENT_TOASTS")
    T.Register_Event("DISPLAY_EVENT_TOASTS", print)
    local function empty_toasts()
        while (true) do
            local toast = C_EventToastManager.GetNextToastToDisplay()
            if toast then
                print(toast.title, toast.subtitle, toast.link)
                C_EventToastManager.RemoveCurrentToast()
            else
                break
            end
        end
    end
    hooksecurefunc(
        EventToastManagerFrame,
        "DisplayToast",
        function(self, ...)
            print("EventToastManagerFrame:DisplayToast", ...)
            self:StopToasting()
            empty_toasts()
        end
    )
    empty_toasts()

    -- 进入副本、升级等屏幕中心提示，副本阶段性提示
    -- LevelUpDisplay_OnLoad = function()end
    -- LevelUpDisplay:UnregisterAllEvents()
    -- LevelUpDisplay.OnEvent = function()end
    -- LevelUpDisplay_OnEvent = function()end
    -- 见LevelUpDisplay_OnLoad()，搜索得到"SCENARIO_UPDATE"事件是在UIParent中处理的
    -- UIParent:UnregisterEvent("SCENARIO_UPDATE")
    -- hooksecurefunc("LevelUpDisplay_OnLoad", function(self)
    --     self:UnregisterAllEvents()
    --     self.PlayBanner = function()end
    --     self.StopBanner = function()end
    --     self.ResumeBanner = function()end
    -- end)
    -- 亲测，只有以下方法有用！
    LevelUpDisplay_StartDisplay = T.Nop

    -- 隐藏8.0艾泽里特升级屏幕中心提示 \Interface\FrameXML\AzeriteItemToasts.xml
    AzeriteLevelUpToast:UnregisterAllEvents()
    AzeriteLevelUpToast.OnEvent = T.Nop
    T.Register_Event("AZERITE_ITEM_POWER_LEVEL_CHANGED", print)

    -- 掷色子奖励隐藏掉
    -- BonusRollFrame_FinishedFading = function()end

    -- 自动进组，隐藏之后的对话框 /Interface/FrameXML/LFGList.lua
    hooksecurefunc(
        LFGListInviteDialog,
        "Show",
        function()
            LFGListInviteDialog.AcceptButton:Click()
        end
    )
    -- 隐藏团队满员退出查找器提示
    LFGListFrame:UnregisterEvent("LFG_LIST_ENTRY_EXPIRED_TOO_MANY_PLAYERS")
    -- 转换为团队对话框自动确定
    T.Hook_StaticPopup(
        "LFG_LIST_AUTO_ACCEPT_CONVERT_TO_RAID",
        function(dialog)
            dialog.button1:Click()
        end
    )

    -- 删除界面各个地方的黄框提示
    local last_helptip_text = ""
    hooksecurefunc(
        HelpTip,
        "Show",
        function(self, parent, info, relativeRegion)
            if info.text ~= last_helptip_text then
                print(info.text)
            end
            last_helptip_text = info.text
            for frame in self.framePool:EnumerateActive() do
                if frame:Matches(parent, info.text) then
                    frame:Acknowledge()
                    break
                end
            end
            -- HelpTip:ForceHideAll()
        end
    )
    for frame in HelpTip.framePool:EnumerateActive() do
        frame:Acknowledge()
    end
    -- HelpTip:ForceHideAll()
    -- 禁止某些界面里的黄框提示
    -- hooksecurefunc(HelpPlateTooltip, "Show", HelpPlateTooltip.Hide)

    -- 公会银行存钱自动填写
    local function auto_fill_guild_bank_money_input(dialog)
        local copper
        if dialog.which == "GUILDBANK_DEPOSIT" then
            copper = GetMoney()
        elseif dialog.which == "GUILDBANK_WITHDRAW" then
            copper = GetGuildBankMoney()
        end
        local g = tostring(math.modf(copper / 100 / 100))
        local s = tostring(math.modf(copper / 100) % 100)
        local c = tostring(copper % 100)
        print(g, s, c)
        dialog.moneyInputFrame.gold:SetText(g)
        dialog.moneyInputFrame.silver:SetText(s)
        dialog.moneyInputFrame.copper:SetText(c)
    end
    T.Hook_StaticPopup("GUILDBANK_DEPOSIT", auto_fill_guild_bank_money_input)
    T.Hook_StaticPopup("GUILDBANK_WITHDRAW", auto_fill_guild_bank_money_input)

    -- 地下城查找器自动拓展搜索范围
    T.Hook_StaticPopup(
        "LFG_QUEUE_EXPAND",
        function(dialog)
            dialog.button1:Click()
        end
    )

    -- 自动确认重置副本
    T.Hook_StaticPopup(
        "CONFIRM_RESET_INSTANCES",
        function(dialog)
            dialog.button1:Click()
        end
    )

    -- 随机地下城和随机战场确认进入对话框
    function delayed_auto_click(dialog, button, secs)
        local ticker = nil
        local countdown = 0
        local cb = function()
            if (not dialog:IsVisible()) or countdown <= 0 then
                ticker:Cancel()
                ticker = nil
                if dialog:IsVisible() and countdown <= 0 then
                    button:Click()
                end
            else
                print(dialog:GetName() .. " " .. countdown)
                countdown = countdown - 1
            end
        end
        hooksecurefunc(
            dialog,
            "Show",
            function(self)
                print(dialog:GetName() .. ":Show()")
                if not ticker then
                    countdown = 6
                    ticker = C_Timer.NewTicker(1, cb)
                    cb()
                end
            end
        )
    end
    delayed_auto_click(LFGDungeonReadyDialog, LFGDungeonReadyDialog.enterButton, 6)
    -- delayed_auto_click(PVPReadyDialog, PVPReadyDialog.enterButton, 6) -- ForceTaint_Strong，估计是防脚本挂机
end
