local N, T = ...

function T.Setup_Friendlist()
    friends = friends or {}
    local my_realm = GetRealmName()
    local _, my_faction = UnitFactionGroup("player")

    local function get_character_friendlist()
        -- 获取角色好友列表
        -- 好友列表应该用GUID而不是角色名作为键，因为角色有可能会改名
        -- print("get_character_friendlist")
        local f = {}
        for index = 1, C_FriendList.GetNumFriends() do
            local info = C_FriendList.GetFriendInfoByIndex(index)
            -- print(info.name, info.notes)
            f[info.guid] = {
                name = info.name,
                notes = info.notes or ""
            }
        end
        return f
    end

    local function sync_character_friendlist_to_account()
        -- 角色好友列表->账号好友列表
        -- print("sync_character_friendlist_to_account")
        friends[my_realm][my_faction] = get_character_friendlist()
    end

    local function sync_account_friendlist_to_character()
        -- 账号好友列表->角色好友列表
        -- print("sync_account_friendlist_to_character")
        local f = get_character_friendlist()
        for guid, info in pairs(friends[my_realm][my_faction]) do
            if not f[guid] then
                print("->AddFriend", info.name, info.notes, C_FriendList.AddFriend(info.name, info.notes))
            elseif f[guid].notes ~= info.notes then
                -- 有时候明明notes不一样却并不执行，/dump friends与磁盘上存储的不一致
                -- 是不是sync_character_friendlist_to_account执行在前了？
                print(f[guid].name, info.notes)
                print(
                    "->SetFriendNotes",
                    f[guid].name,
                    f[guid].notes,
                    "->",
                    info.notes,
                    C_FriendList.SetFriendNotes(f[guid].name, info.notes or "")
                )
            elseif f[guid].name ~= info.name then
                -- 好友可能会更名
                friends[my_realm][my_faction][guid].name = f[guid].name
            end
        end
        for guid, info in pairs(f) do
            if not friends[my_realm][my_faction][guid] then
                print("->RemoveFriend", info.name, C_FriendList.RemoveFriend(info.name))
            end
        end
        -- 清理账号好友列表中已经不存在的
        -- C_FriendList的增删改操作都有延迟，下面的代码会AddFriend之后还没完成的情况下执行，出现误删
        -- local to_remove = {}
        -- for guid, info in pairs(friends[my_realm][my_faction]) do
        --     if not C_FriendList.IsFriend(guid) then
        --         table.insert(to_remove, guid)
        --     end
        -- end
        -- for _, guid in ipairs(to_remove) do
        --     friends[my_realm][my_faction][guid] = nil
        -- end
    end

    local function initialize()
        T.Unregister_Event("FRIENDLIST_UPDATE", initialize)
        if not friends[my_realm] then
            friends[my_realm] = {}
        end
        if not friends[my_realm][my_faction] then
            -- 如果账号好友列表为空（第一个登录的角色）那么以该角色好友列表作为账号列表
            StaticPopupDialogs["COPY_THIS_CHARACTER_FRIENDLIST_TO_ACCOUNT_WIDE"] = {
                text = "是否以当前角色的好友列表作为基准，覆盖其它所有角色的好友列表？注意！该操作只会执行一次，请务必选好作为基准的角色。",
                button1 = "确定",
                button2 = "取消",
                OnAccept = function()
                    sync_character_friendlist_to_account()
                end,
                timeout = 0,
                whileDead = true,
                hideOnEscape = true
            }
            StaticPopup_Show("COPY_THIS_CHARACTER_FRIENDLIST_TO_ACCOUNT_WIDE")
        else
            sync_account_friendlist_to_character()
        end

        -- 处理用户的增删改操作，同样要放在FRIENDLIST_UPDATE事件中执行
        -- PLAYER_LOGOUT事件中执行get_character_friendlist()会失败
        local _cf = get_character_friendlist()
        T.Register_Event(
            "FRIENDLIST_UPDATE",
            function()
                _cf = get_character_friendlist()
            end
        )
        T.Register_Event(
            "PLAYER_LOGOUT",
            function()
                -- 账户角色列表如果不存在，表明该账户尚未进行第一次同步，此时角色的增删改操作不应该同步到账户
                if friends[my_realm][my_faction] then
                    friends[my_realm][my_faction] = _cf
                end
            end
        )
    end

    -- 注意！角色登录进游戏，如果没有打开好友列表，那么本地的好友信息是空的，C_FriendList.GetNumFriends()等函数没有意义
    -- 所以需要主动通过C_FriendList.ShowFriends()向服务器获取好友信息，触发FRIENDLIST_UPDATE事件，在该事件里完成初始化工作
    T.Register_Event("FRIENDLIST_UPDATE", initialize)
    C_FriendList.ShowFriends()
end
